//+------------------------------------------------------------------+
//|                                                 IndStdDev_v1.mq5 |
//|                        Copyright 2018, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2018, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_plots 3
#property indicator_type1 DRAW_LINE
#property indicator_type2 DRAW_LINE
#property indicator_type3 DRAW_LINE
#property indicator_style1 STYLE_SOLID
#property indicator_style2 STYLE_SOLID
#property indicator_style3 STYLE_SOLID
#property indicator_color1 clrMediumSeaGreen
#property indicator_color2 clrMediumSlateBlue
#property indicator_color3 clrMediumSlateBlue
#property indicator_width1 1
#property indicator_width2 1
#property indicator_width3 1
#property indicator_label1 "StdDev"
#property indicator_label2 "Level"
#property indicator_label3 "Smoothing"
//----
#include <MovingAverages.mqh>
//----
input int           StdDevPeriod =20;
input int            StdDevShift =0;
input ENUM_MA_METHOD    MaMethod =MODE_SMA;
input ENUM_APPLIED_PRICE MaPrice =PRICE_CLOSE;
input double                   R =25.0;
input int                      D =100;
input int               PeriodMa =14;
input ENUM_MA_METHOD    MethodMa =MODE_SMA;
//----
double ExtStdDevBuffer[];
double ExtLevelBuffer[];
double ExtSmoothingBuffer[];
double buff_dbl[];
//----
int h_sd;
static int ws;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits+1);
   IndicatorSetString(INDICATOR_SHORTNAME,"StdDev("+string(StdDevPeriod)+")");
   SetIndexBuffer(0,ExtStdDevBuffer);
   SetIndexBuffer(1,ExtLevelBuffer);
   PlotIndexSetInteger(1,PLOT_DRAW_TYPE,DRAW_NONE);
   SetIndexBuffer(2,ExtSmoothingBuffer);
   ArraySetAsSeries(ExtStdDevBuffer,true);
   ArraySetAsSeries(ExtLevelBuffer,true);
   ArraySetAsSeries(ExtSmoothingBuffer,true);
//----
   h_sd=iStdDev(_Symbol,_Period,StdDevPeriod,StdDevShift,MaMethod,MaPrice);
//----
   if(h_sd==-1)
      return(INIT_FAILED);
//----
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   int limit=rates_total-prev_calculated;
//----
   if(prev_calculated==0)
      limit=rates_total-1;
//----
   for(int i=limit;i>=0;i--)
     {
      if(CopyBuffer(h_sd,0,i,1,buff_dbl)!=1)
         return(0);
      //----
      ExtStdDevBuffer[i]=buff_dbl[0];
     }
//----
   int cnt_hits=int((D/100.0)*R);
//----
   for(int i=limit;i>=0;i--)
     {
      if(i+D>=rates_total)
         continue;
      //----
      double level_curr=0.0;
      double level_prev=0.0;
      //----
      for(int j=0;j<D;j++)
        {
         for(int k=0;k<D;k++)
            if((ExtStdDevBuffer[i+k]<level_curr
               || level_curr==0.0)
               && ExtStdDevBuffer[i+k]>level_prev)
               level_curr=ExtStdDevBuffer[i+k];
         //----
         int cnt=0;
         //----
         for(int k=0;k<D;k++)
            if(ExtStdDevBuffer[i+k]<=level_curr)
               cnt++;
         //----
         if(cnt>cnt_hits)
            break;
         //----
         level_prev=level_curr;
         level_curr=0.0;
        }
      //----
      ExtLevelBuffer[i]=(level_curr+level_prev)/2.0;
     }
//----
   switch(MethodMa)
     {
      case MODE_SMA:
        {
         SimpleMAOnBuffer(rates_total,prev_calculated,MathMax(StdDevPeriod,PeriodMa),PeriodMa,ExtLevelBuffer,ExtSmoothingBuffer);
         break;
        }
      //----
      case MODE_EMA:
        {
         ExponentialMAOnBuffer(rates_total,prev_calculated,MathMax(StdDevPeriod,PeriodMa),PeriodMa,ExtLevelBuffer,ExtSmoothingBuffer);
         break;
        }
      //----
      case MODE_LWMA:
        {
         LinearWeightedMAOnBuffer(rates_total,prev_calculated,MathMax(StdDevPeriod,PeriodMa),PeriodMa,ExtLevelBuffer,ExtSmoothingBuffer,ws);
         break;
        }
      //----
      case MODE_SMMA:
        {
         SmoothedMAOnBuffer(rates_total,prev_calculated,MathMax(StdDevPeriod,PeriodMa),PeriodMa,ExtLevelBuffer,ExtSmoothingBuffer);
         break;
        }
      //----
      default:
         SimpleMAOnBuffer(rates_total,prev_calculated,MathMax(StdDevPeriod,PeriodMa),PeriodMa,ExtLevelBuffer,ExtSmoothingBuffer);
     }
//----
   return(rates_total);
  }
//+------------------------------------------------------------------+
